@@ -15,16 +15,14 @@ package MCE::Core::Input::Generator;
 use strict;
 use warnings;
 
-our $VERSION = '1.600';
+our $VERSION = '1.601';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
 no warnings 'threads';
+no warnings 'recursion';
 no warnings 'uninitialized';
 
 ###############################################################################
@@ -128,10 +126,10 @@ sub _worker_sequence_generator {
                   );
                   $_start = 1 if ($_start < 1);
 
-                  for ($_start .. $_chunk_size) {
+                  for my $_i ($_start .. $_chunk_size) {
                      last if ($_next > $_end);
                      $_tmp_e = $_next;
-                     $_next  = $_step * $_ + $_n_begin;
+                     $_next  = $_step * $_i + $_n_begin;
                   }
                }
             }
@@ -145,10 +143,10 @@ sub _worker_sequence_generator {
                   );
                   $_start = 1 if ($_start < 1);
 
-                  for ($_start .. $_chunk_size) {
+                  for my $_i ($_start .. $_chunk_size) {
                      last if ($_next < $_end);
                      $_tmp_e = $_next;
-                     $_next  = $_step * $_ + $_n_begin;
+                     $_next  = $_step * $_i + $_n_begin;
                   }
                }
             }
@@ -174,24 +172,24 @@ sub _worker_sequence_generator {
                   }
                }
                else {
-                  for (1 .. $_chunk_size) {
+                  for my $_i (1 .. $_chunk_size) {
                      last if ($_next > $_end);
 
                      push @_n, (defined $_fmt)
                         ? sprintf("%$_fmt", $_next) : $_next;
 
-                     $_next = $_step * $_ + $_n_begin;
+                     $_next = $_step * $_i + $_n_begin;
                   }
                }
             }
             else {
-               for (1 .. $_chunk_size) {
+               for my $_i (1 .. $_chunk_size) {
                   last if ($_next < $_end);
 
                   push @_n, (defined $_fmt)
                      ? sprintf("%$_fmt", $_next) : $_next;
 
-                  $_next = $_step * $_ + $_n_begin;
+                  $_next = $_step * $_i + $_n_begin;
                }
             }
 
