@@ -14,20 +14,18 @@ package MCE::Core::Input::Handle;
 use strict;
 use warnings;
 
-our $VERSION = '1.600';
+our $VERSION = '1.601';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use bytes;
-
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
 no warnings 'threads';
+no warnings 'recursion';
 no warnings 'uninitialized';
 
+use bytes;
+
 my $_que_read_size = $MCE::_que_read_size;
 my $_que_template  = $MCE::_que_template;
 
