@@ -16,20 +16,18 @@ use warnings;
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-our $VERSION = '1.600';
+our $VERSION = '1.601';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use bytes;
-
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
 no warnings 'threads';
+no warnings 'recursion';
 no warnings 'uninitialized';
 
+use bytes;
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Call on task_end after task completion.
@@ -85,8 +83,7 @@ sub _output_loop {
       $_input_size, $_offset_pos, $_single_dim, @_gather, $_cs_one_flag,
       $_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_len, $_sync_cnt,
       $_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
-      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT,
-      $_rla_chunkid, $_rla_nextid
+      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT
    );
 
    ## -------------------------------------------------------------------------
@@ -171,18 +168,6 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_W_RLA.$LF => sub {                   ## Worker has relayed
-         my ($_chunk_id, $_next_id) = split(':', <$_DAU_R_SOCK>);
-
-         if ($_chunk_id > $_rla_chunkid) {
-            chomp $_next_id;
-            $_rla_chunkid = $_chunk_id;
-            $_rla_nextid  = $_next_id;
-         }
-
-         return;
-      },
-
       ## ----------------------------------------------------------------------
 
       OUTPUT_W_EXT.$LF => sub {                   ## Worker has exited
@@ -721,16 +706,16 @@ sub _output_loop {
    }
 
    if ($_has_user_tasks) {
-      foreach (0 .. @{ $self->{user_tasks} } - 1) {
-         $_gather[$_] = (defined $self->{user_tasks}->[$_]->{gather})
-            ? $self->{user_tasks}->[$_]->{gather} : $self->{gather};
+      for my $_i (0 .. @{ $self->{user_tasks} } - 1) {
+         $_gather[$_i] = (defined $self->{user_tasks}->[$_i]->{gather})
+            ? $self->{user_tasks}->[$_i]->{gather} : $self->{gather};
 
-         $_is_c_ref[$_] = ( ref $_gather[$_] eq 'CODE' ) ? 1 : 0;
-         $_is_h_ref[$_] = ( ref $_gather[$_] eq 'HASH' ) ? 1 : 0;
+         $_is_c_ref[$_i] = ( ref $_gather[$_i] eq 'CODE' ) ? 1 : 0;
+         $_is_h_ref[$_i] = ( ref $_gather[$_i] eq 'HASH' ) ? 1 : 0;
 
-         $_is_q_ref[$_] = (
-            ref $_gather[$_] eq 'MCE::Queue' ||
-            ref $_gather[$_] eq 'Thread::Queue' ) ? 1 : 0;
+         $_is_q_ref[$_i] = (
+            ref $_gather[$_i] eq 'MCE::Queue' ||
+            ref $_gather[$_i] eq 'Thread::Queue' ) ? 1 : 0;
       }
    }
 
@@ -801,27 +786,8 @@ sub _output_loop {
    $_I_FLG  = (!$_I_SEP || $_I_SEP ne $LF) ? 1 : 0;
 
    ## Call module's loop_begin routine for modules plugged into MCE.
-   $_->($self, \$_DAU_R_SOCK) for (@{ $_plugin_loop_begin });
-
-   ## Write initial values for relaying.
-   if (defined $self->{init_relay}) {
-      my $_RLA_W_SOCK = $self->{_rla_w_sock}->[0];
-      my $_init_relay;
-
-      if (ref $self->{init_relay} eq '') {
-         $_init_relay = $self->{init_relay} . '0';
-      }
-      elsif (ref $self->{init_relay} eq 'HASH') {
-         $_init_relay = $self->{freeze}($self->{init_relay}) . '1';
-      }
-      elsif (ref $self->{init_relay} eq 'ARRAY') {
-         $_init_relay = $self->{freeze}($self->{init_relay}) . '2';
-      }
-
-      print {$_RLA_W_SOCK} length($_init_relay) . $LF . $_init_relay;
-      delete $self->{_rla_return} if (exists $self->{_rla_return});
-
-      $_rla_chunkid = $_rla_nextid = 0;
+   for my $_p (@{ $_plugin_loop_begin }) {
+      $_p->($self, \$_DAU_R_SOCK);
    }
 
    ## Call on hash function. Exit loop when workers have completed.
@@ -841,31 +807,19 @@ sub _output_loop {
       last unless ($self->{_total_running});
    }
 
-   ## Obtain final relay values.
-   if (defined $self->{init_relay}) {
-      my $_RLA_R_SOCK = $self->{_rla_r_sock}->[$_rla_nextid];
-      my ($_len, $_ret); chomp($_len = <$_RLA_R_SOCK>);
-
-      read $_RLA_R_SOCK, $_ret, $_len;
-
-      if (chop $_ret) {
-         $self->{_rla_return} = $self->{thaw}($_ret);
-      } else {
-         $self->{_rla_return} = $_ret;
-      }
-   }
-
    ## Call module's loop_end routine for modules plugged into MCE.
-   $_->($self) for (@{ $_plugin_loop_end });
+   for my $_p (@{ $_plugin_loop_end }) {
+      $_p->($self);
+   }
 
    ## Call on_post_run callback.
    $_on_post_run->($self, $self->{_status}) if (defined $_on_post_run);
 
    ## Close opened sendto file handles.
-   for (keys %_sendto_fhs) {
-      close  $_sendto_fhs{$_};
-      undef  $_sendto_fhs{$_};
-      delete $_sendto_fhs{$_};
+   for my $_p (keys %_sendto_fhs) {
+      close  $_sendto_fhs{$_p};
+      undef  $_sendto_fhs{$_p};
+      delete $_sendto_fhs{$_p};
    }
 
    ## Restore the default handle. Close MCE STDOUT/STDERR handles.
