@@ -18,7 +18,9 @@ use Scalar::Util qw( looks_like_number );
 use MCE;
 use MCE::Queue;
 
-our $VERSION = '1.600';
+our $VERSION  = '1.601';
+
+our @CARP_NOT = qw( MCE );
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -66,8 +68,8 @@ sub import {
          $MCE::TMP_DIR = $MCE::MCE->{tmp_dir} = shift;
          my $_e1 = 'is not a directory or does not exist';
          my $_e2 = 'is not writeable';
-         _croak("$_tag::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
-         _croak("$_tag::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
          next;
       }
 
@@ -76,7 +78,7 @@ sub import {
          next;
       }
 
-      _croak("$_tag::import: ($_argument) is not a valid module argument");
+      _croak($_tag."::import: ($_argument) is not a valid module argument");
    }
 
    _croak("$_tag: (DEFAULT_MODE) is not valid")
@@ -180,7 +182,7 @@ sub finish () {
    $_gather_ref = $_order_id = undef; undef %_tmp; @_user_tasks = ();
    @_prev_w = (); @_prev_n = (); @_prev_m = (); @_prev_c = ();
 
-   $_->DESTROY() foreach (@_queue); @_queue = ();
+   $_->DESTROY() for (@_queue); @_queue = ();
 
    return;
 }
@@ -205,10 +207,10 @@ sub run_file (@) {
       $_params = {};
    }
 
-   for ($_start_pos .. @_ - 1) {
-      my $_r = ref $_[$_];
+   for my $_i ($_start_pos .. @_ - 1) {
+      my $_r = ref $_[$_i];
       if ($_r eq '' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
-         $_file = $_[$_]; $_pos = $_;
+         $_file = $_[$_i]; $_pos = $_i;
          last;
       }
    }
@@ -254,11 +256,11 @@ sub run_seq (@) {
       $_params = {};
    }
 
-   for ($_start_pos .. @_ - 1) {
-      my $_ref = ref $_[$_];
+   for my $_i ($_start_pos .. @_ - 1) {
+      my $_ref = ref $_[$_i];
 
       if ($_ref eq '' || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
-         $_pos = $_;
+         $_pos = $_i;
 
          if ($_ref eq '') {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
@@ -315,7 +317,9 @@ sub run (@) {
 
    if (ref $_[0] eq 'HASH' && !exists $_[0]->{code}) {
       $_params = {} unless defined $_params;
-      $_params->{$_} = $_[0]->{$_} foreach (keys %{ $_[0] });
+      for my $_p (keys %{ $_[0] }) {
+         $_params->{$_p} = $_[0]->{$_p};
+      }
 
       shift;
    }
@@ -404,7 +408,7 @@ sub run (@) {
       delete $_p->{gather}      if (exists $_p->{gather});
    }
 
-   if (@_code > 1) {
+   if (@_code > 1 && $_max_workers > 1) {
       $_max_workers = int($_max_workers / @_code + 0.5) + 1;
    }
 
@@ -442,9 +446,9 @@ sub run (@) {
       );
 
       if (defined $_params) {
-         my $_p = $_params;
+         local $_; my $_p = $_params;
 
-         foreach (keys %{ $_p }) {
+         for (keys %{ $_p }) {
             next if ($_ eq 'sequence_run');
             next if ($_ eq 'max_workers' && ref $_p->{max_workers} eq 'ARRAY');
             next if ($_ eq 'task_name' && ref $_p->{task_name} eq 'ARRAY');
@@ -465,12 +469,12 @@ sub run (@) {
       ## Workers may persist after running. Thus, updating the MCE instance.
       ## These options do not require respawning.
       if (defined $_params) {
-         for (qw(
+         for my $_p (qw(
             RS interval stderr_file stdout_file user_error user_output
             job_delay submit_delay on_post_exit on_post_run user_args
             flush_file flush_stderr flush_stdout
          )) {
-            $_MCE->{$_} = $_params->{$_} if (exists $_params->{$_});
+            $_MCE->{$_p} = $_params->{$_p} if (exists $_params->{$_p});
          }
       }
    }
@@ -660,7 +664,7 @@ MCE::Stream - Parallel stream model for chaining multiple maps and greps
 
 =head1 VERSION
 
-This document describes MCE::Stream version 1.600
+This document describes MCE::Stream version 1.601
 
 =head1 SYNOPSIS
 
